﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Styx;
using Styx.Common;
using Styx.CommonBot;
using Styx.CommonBot.Routines;
using Styx.Helpers;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Action = Styx.TreeSharp.Action;
using CommonBehaviors.Actions;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace Warena
{
    public partial class Routine
    {
        #region BattleShout()
        public Composite BattleShout()
        {
            return new Decorator(
                ret =>
                    IsAttackable(StyxWoW.Me.CurrentTarget) &&
                    SpellManager.CanCast("Battle Shout") &&
                    (
                        StyxWoW.Me.RagePercent < 30 && StyxWoW.Me.CurrentTarget.HealthPercent <= 20 ||
                        StyxWoW.Me.RagePercent < 30 && NeedTasteForBloodDump ||
                        !StyxWoW.Me.HasAura("Battle Shout")
                    ),

                new Action(delegate
                {
                    SpellManager.Cast("Battle Shout");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Battle Shout");
                }
                )
            );
        }
        #endregion
        #region BattleShoutSF()
        public Composite BattleShoutSF()
        {
            return new Decorator(
                ret =>
                    EnemyIsInvulnerable30.Count() > 1 &&
                    StyxWoW.Me.RagePercent < 30 &&
                    SpellManager.CanCast("Battle Shout"),

                new Action(delegate
                {
                    SpellManager.Cast("Battle Shout");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Battle Shout For Rage:" + EnemyIsInvulnerable30.FirstOrDefault().Name + " is invulnerable.");
                }
                )
            );
        }
        #endregion
        #region BattleStance()
        public Composite BattleStance()
        {
            return new Decorator(ret => StyxWoW.Me.HealthPercent >= 30 && !StyxWoW.Me.HasAura("Battle Stance") && SpellManager.CanCast("Battle Stance"),
                        new Action(delegate
                        {
                            SpellManager.Cast("Battle Stance");
                            Logging.Write(StyxWoW.Me.HealthPercent + "% - Battle Stance");
                        }
                        )
            );
        }
        #endregion
        #region BerserkerRageForSF()
        public Composite BerserkerRageForSF()
        {
            return new Decorator(
                ret =>
                    EnemyIsInvulnerable30.Count() > 1 &&
                    StyxWoW.Me.RagePercent < 30 &&
                    StyxWoW.Me.RagePercent >= 20 &&
                    SpellManager.CanCast("Berserker Rage"),

                new Action(delegate
                {
                    SpellManager.Cast("Berserker Rage");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Berserker Rage For Rage:" + EnemyIsInvulnerable30.FirstOrDefault().Name + " is invulnerable.");
                }
                )
            );
        }
        #endregion
        #region Cast(spell, reqs)

        private delegate T Selection<out T>(object context);

        Composite Cast(string spell, Selection<bool> reqs = null)
        {
            return
                new Decorator(
                    ret => ((reqs != null && reqs(ret)) || (reqs == null)) && SpellManager.CanCast(spell),
                new Action(
                    ret => 
                        SpellManager.Cast(spell)
                ));

        }
        #endregion
        #region Cleave()
        public Composite Cleave()
        {
            return new Decorator(
                ret =>
                    SpellManager.CanCast("Cleave") &&
                    StyxWoW.Me.RagePercent > 30 &&
                    IsAttackable(StyxWoW.Me.CurrentTarget) &&
                    StyxWoW.Me.CurrentTarget.IsWithinMeleeRange &&
                    EnemyUnitsInMelee.Count() >= 2 &&
                    EnemyCCInMelee.Count() < 1,

                new Action(delegate
                {
                    SpellManager.Cast("Cleave");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Cleave");
                }
                )
            );
        }
        #endregion
        #region DefensiveStance()
        public Composite DefensiveStance()
        {
            return new Decorator(ret => StyxWoW.Me.HealthPercent <= 30 && !StyxWoW.Me.HasAura("Defensive Stance") && SpellManager.CanCast("Defensive Stance"),
                        new Action(delegate
                        {
                            SpellManager.Cast("Defensive Stance");
                            Logging.Write(StyxWoW.Me.HealthPercent + "% - Defensive Stance");
                        }
                        )
            );
        }
        #endregion
        #region DemoralizingBanner()
        public Composite DemoralizingBannerMe()
        {
            return new Decorator(ret => 
                    StyxWoW.Me.HealthPercent < 25 && 
                    SpellManager.CanCast("Demoralizing Banner"),
                new Sequence(
                    new Action(delegate
                        {
                            SpellManager.Cast("Demoralizing Banner");
                            SpellManager.ClickRemoteLocation(StyxWoW.Me.Location);
                            Logging.Write("Casting Demoralizing Banner: " + StyxWoW.Me.Name.ToString());
                        }
                    )
                )
            );
        }
        #endregion
        #region DemoralizingBannerFriend()
        public Composite DemoralizingBannerFriend()
        {
            return new Decorator(
                ret => 
                    InArena &&
                    FriendsNeedingHelp30.Count() >= 1 &&
                    SpellManager.CanCast("Demoralizing Banner"),
                new Sequence(
                    new Action(delegate
                        {
                            SpellManager.Cast("Demoralizing Banner");
                            SpellManager.ClickRemoteLocation(FriendsNeedingHelp30.FirstOrDefault().Location);
                            Logging.Write("Casting Demoralizing Banner: " + FriendsNeedingHelp30.FirstOrDefault().Name.ToString());
                        }
                    )
                )
            );
        }
        #endregion
        #region DemoralizingShout()
        public Composite DemoralizingShout()
        {
            return new Decorator(
                ret =>
                IsAttackable(StyxWoW.Me.CurrentTarget) &&
                StyxWoW.Me.CurrentTarget.Distance <= 10 &&
                StyxWoW.Me.CurrentTarget.InLineOfSpellSight &&
                !StyxWoW.Me.CurrentTarget.ActiveAuras.ContainsKey("Demoralizing Shout") &&
                SpellManager.CanCast("Demoralizing Shout"),

                new Action(delegate
                {
                    SpellManager.Cast("Demoralizing Shout");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Demoralizing Shout");
                }
                )
            );
        }
        #endregion
        #region Devastate()
        public Composite Devastate()
        {
            return new Decorator(
                ret =>
                IsAttackable(StyxWoW.Me.CurrentTarget) &&
                StyxWoW.Me.CurrentTarget.IsWithinMeleeRange &&
                StyxWoW.Me.CurrentTarget.HealthPercent > 20 &&
                SpellManager.CanCast("Devastate"),

                new Action(delegate
                {
                    SpellManager.Cast("Devastate");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Devastate");
                }
                )
            );
        }
        #endregion       
        #region DieByTheSword()
        public Composite DieByTheSword()
        {
            return new Decorator(
                ret =>
                    IsAttackable(StyxWoW.Me.CurrentTarget) &&
                    StyxWoW.Me.CurrentTarget.IsWithinMeleeRange &&
                    StyxWoW.Me.Combat &&
                    !StyxWoW.Me.ActiveAuras.ContainsKey("Shield Wall") &&
                    SpellManager.CanCast("Die by the Sword") &&
                    StyxWoW.Me.HealthPercent <= 25,
                        new Action(delegate
                        {
                            SpellManager.Cast("Die By The Sword");
                            Logging.Write(StyxWoW.Me.HealthPercent + "% - Die By The Sword");
                        }
                        )
            );
        }
        #endregion
        #region Disarm()
        public Composite Disarm()
        {
            return new Decorator(
                ret =>
                    IsAttackable(StyxWoW.Me.CurrentTarget) &&
                    StyxWoW.Me.CurrentTarget.IsPlayer &&
                    StyxWoW.Me.CurrentTarget.IsWithinMeleeRange &&
                    SpellManager.CanCast("Disarm") &&
                    StyxWoW.Me.CurrentTarget.Class != WoWClass.Druid &&
                    (
                        (StyxWoW.Me.HealthPercent <= 25 || FriendsNeedingHelp30.Count() >= 1) ||
                        (StyxWoW.Me.CurrentTarget.Class == WoWClass.Warrior && StyxWoW.Me.CurrentTarget.HasAura("Avatar")) ||
                        (StyxWoW.Me.CurrentTarget.Class == WoWClass.Warrior && StyxWoW.Me.CurrentTarget.HasAura("Bloodbath")) ||
                        (StyxWoW.Me.CurrentTarget.Class == WoWClass.Warrior && StyxWoW.Me.CurrentTarget.HasAura("Recklessness")) ||
                        (StyxWoW.Me.CurrentTarget.Class == WoWClass.Rogue && StyxWoW.Me.CurrentTarget.HasAura("Shadow Dance")) ||
                        (StyxWoW.Me.CurrentTarget.Class == WoWClass.Rogue && StyxWoW.Me.CurrentTarget.HasAura("Killing Spree")) ||
                        (StyxWoW.Me.CurrentTarget.Class == WoWClass.DeathKnight && StyxWoW.Me.CurrentTarget.HasAura("Vampiric Blood"))
                    ),
                new Action(delegate
                {
                    SpellManager.Cast("Disarm");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Disarmed: " + StyxWoW.Me.CurrentTarget.Name);
                }
                )
            );
        }
        #endregion
        #region DisruptingShout()
        public Composite DisruptingShout()
        {
            return new Decorator(
                ret =>
                    SpellManager.CanCast("Disrupting Shout") &&
                    EnemyCastingCC10.Count() >= 1,

                new Action(delegate
                {
                    SpellManager.Cast("Disrupting Shout");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Disrupting Shout");
                }
                )
            );
        }
        #endregion
        #region doMassSpellReflect

        bool doMassSpellRefect = EnemiesCastingCrowdControl60.Any(e => nearbyFriendlies20.Contains(e.CurrentTarget));

        #endregion
        #region Equip2H()
        public Composite Equip2H()
        {
            return
                new Decorator(
                ret =>
                StyxWoW.Me.Combat &&
                StyxWoW.Me.HealthPercent > 75 &&
                !StyxWoW.Me.CurrentTarget.HasAura("Spell Reflection") &&
                !StyxWoW.Me.CurrentTarget.HasAura("Shield Wall") &&
                StyxWoW.Me.Inventory.Equipped.OffHand != null,
                    new Action(delegate
                    {
                        Lua.DoString("RunMacroText(\"/equipslot 16 " + "Malevolent Gladiator's Decapitator" + "\")");
                        //return RunStatus.Success;
                    }
                    )
            );
        }
        #endregion
        #region Execute()
        public Composite Execute()
        {
            return new Decorator(
                ret =>
                    IsAttackable(StyxWoW.Me.CurrentTarget) &&
                    !IsImmune(StyxWoW.Me.CurrentTarget) &&
                    StyxWoW.Me.CurrentTarget.IsWithinMeleeRange &&
                    StyxWoW.Me.CurrentTarget.HealthPercent <= 20 &&
                    StyxWoW.Me.RagePercent >= 30,

                new Action(delegate
                {
                    SpellManager.Cast("Execute");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Execute on " + StyxWoW.Me.CurrentTarget.Name + " at " + StyxWoW.Me.CurrentTarget.HealthPercent + "%.");
                }
                )
            );
        }
        #endregion
        #region GetKeyState
        [DllImport("user32.dll")]
        internal static extern short GetAsyncKeyState(Keys vKey);
        #endregion
        #region GetSpellCooldown()
        public static TimeSpan GetSpellCooldown(string spell)
        {
            SpellFindResults results;
            if (SpellManager.FindSpell(spell, out results))
            {
                if (results.Override != null)
                    return results.Override.CooldownTimeLeft;
                return results.Original.CooldownTimeLeft;
            }


            return TimeSpan.MaxValue;
        }

        #endregion
        #region Hamstring()
        public Composite Hamstring()
        {
            return new Decorator(
                ret =>
                //IsAttackable(StyxWoW.Me.CurrentTarget) &&
                //!IsImmune(StyxWoW.Me.CurrentTarget) &&
                //!IsInCC(StyxWoW.Me.CurrentTarget) &&
                //StyxWoW.Me.CurrentTarget.IsWithinMeleeRange &&
                //StyxWoW.Me.CurrentTarget.HealthPercent > 20 &&
                //StyxWoW.Me.CurrentTarget.IsPlayer &&
                !NeedTasteForBloodDump &&
                !StyxWoW.Me.CurrentTarget.HasAura("Hamstring") &&
                !StyxWoW.Me.CurrentTarget.HasAura("Piercing Howl") &&
                !StyxWoW.Me.CurrentTarget.HasAura("Crippling Poison") &&
                !StyxWoW.Me.CurrentTarget.HasAura("Hand of Freedom") &&
                !StyxWoW.Me.CurrentTarget.HasAura("Sluggish") &&
                !StyxWoW.Me.CurrentTarget.HasAura("Infected Wounds"),

                new Action(delegate
                {
                    SpellManager.Cast("Hamstring");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Hamstring");
                }
                )
            );
        }
        #endregion
        #region HasTFB
        bool HasTFB
        {
            get
            {
                var htfb = StyxWoW.Me.GetAuraByName("Taste for Blood");
                if (htfb != null)
                {
                    if (htfb.StackCount >= 1)
                        return true;
                }
                return false;
            }
        }
        #endregion
        #region HasWeakenedArmor
        bool HasWeakenedArmor
        {
            get
            {
                var wa = StyxWoW.Me.CurrentTarget.GetAuraByName("Weakened Armor");
                if (wa != null)
                {
                    if (wa.StackCount == 3 && wa.TimeLeft.Seconds > 2)
                        return true;
                }
                return false;
            }
        }
        #endregion
        #region HeroicLeap()
        WoWPoint preLeap;
        public Composite HeroicLeap()
        {
            return new Decorator(ret =>
                IsAttackable(StyxWoW.Me.CurrentTarget) &&
                !IsCrowdControlled(StyxWoW.Me) &&
                !IsMovementImpaired(StyxWoW.Me) &&
                SpellManager.CanCast("Heroic Leap") &&
                //!SpellManager.CanCast("Charge") && 
                //SpellManager.Spells["Charge"].CooldownTimeLeft.Seconds < 10 &&
                //!StyxWoW.Me.CurrentTarget.HasAura("Charge Stun") &&
                //StyxWoW.Me.IsFacing(StyxWoW.Me.CurrentTarget) &&
                StyxWoW.Me.CurrentTarget.Distance > 8 &&
                StyxWoW.Me.CurrentTarget.Distance < 40 &&
                StyxWoW.Me.CurrentTarget.InLineOfSpellSight,

            new Sequence(
                new Action(delegate
                {
                    preLeap = StyxWoW.Me.Location;
                    SpellManager.Cast("Heroic Leap");
                    SpellManager.ClickRemoteLocation(StyxWoW.Me.CurrentTarget.Location);
                    new WaitContinue(TimeSpan.FromMilliseconds(500), wc => preLeap.Distance(StyxWoW.Me.Location) < 2f, new ActionAlwaysFail());
                    Logging.Write(StyxWoW.Me.HealthPercent + "% Heroic Leap To : " + StyxWoW.Me.CurrentTarget.Name.ToString());
                })
            ));
        }


        #endregion
        #region HeroicStrikeDeadlyCalm()
        public Composite HeroicStrikeDeadlyCalm()
        {
            return new Decorator(
                ret =>
                IsAttackable(StyxWoW.Me.CurrentTarget) &&
                StyxWoW.Me.CurrentTarget.IsWithinMeleeRange &&
                StyxWoW.Me.CurrentTarget.HealthPercent > 20 &&
                StyxWoW.Me.HasAura("Deadly Calm") &&
                SpellManager.CanCast("Heroic Strike"),

                new Action(delegate
                {
                    SpellManager.Cast("Heroic Strike");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Heroic Strike - Deadly Calm");
                }
                )
            );
        }
        #endregion
        #region HeroicStrikeUltimatum()
        public Composite HeroicStrikeUltimatum()
        {
            return new Decorator(
                ret =>
                IsAttackable(StyxWoW.Me.CurrentTarget) &&
                StyxWoW.Me.CurrentTarget.IsWithinMeleeRange &&
                StyxWoW.Me.CurrentTarget.HealthPercent > 20 &&
                StyxWoW.Me.HasAura(122510) &&
                SpellManager.CanCast("Heroic Strike"),

                new Action(delegate
                {
                    SpellManager.Cast("Heroic Strike");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Heroic Strike - Ultimatum");
                }
                )
            );
        }
        #endregion
        #region HeroicThrow()
        public Composite HeroicThrow()
        {
            return new Decorator(
                ret =>
                    IsAttackable(StyxWoW.Me.CurrentTarget) &&
                    StyxWoW.Me.CurrentTarget.InLineOfSpellSight &&
                    SpellManager.CanCast("Heroic Throw") &&
                    !StyxWoW.Me.CurrentTarget.IsWithinMeleeRange &&
                    StyxWoW.Me.CurrentTarget.HealthPercent <= 5,

                new Action(delegate
                {
                    SpellManager.Cast("Heroic Throw");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Heroic Throw on " + StyxWoW.Me.CurrentTarget.Name + " at " + StyxWoW.Me.CurrentTarget.HealthPercent + "%.");
                }
                )
            );
        }
        #endregion
        #region IntimidatingShout()
        public Composite IntimidatingShout()
        {
            return new Decorator(
                ret =>
                    EnemyDistance8.Count() > 1 && SpellManager.CanCast("Intimidating Shout"),
                new Action(delegate
                {
                    //StyxWoW.Me.Target(EnemyDistance8.FirstOrDefault());
                    EnemyDistance8.FirstOrDefault().Target();
                    SpellManager.Cast("Intimidating Shout");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Cast Intimidating Shout" + EnemyDistance8.FirstOrDefault().Name);
                })
            );
        }
        #endregion
        #region InArena
        bool InArena
        {
            get
            {
                if (StyxWoW.Me.ZoneId == 3702) { return true; }
                if (StyxWoW.Me.ZoneId == 4378) { return true; }
                if (StyxWoW.Me.ZoneId == 3698) { return true; }
                if (StyxWoW.Me.ZoneId == 3968) { return true; }
                if (StyxWoW.Me.ZoneId == 4406) { return true; }
                if (StyxWoW.Me.ZoneId == 6296) { return true; }
                return false;
            }
        }
        #endregion
        #region IsArms
        bool IsAms
        {
            get
            {
                if (!SpellManager.HasSpell("Mortal Strike"))
                {
                    return true;
                }
                return false;
            }
        }
        #endregion
        #region LosingTasteForBlood
        bool LosingTasteForBlood
        {
            get
            {
                var tfb = StyxWoW.Me.GetAuraByName("Taste for Blood");
                if (tfb != null)
                {
                    if (tfb.StackCount >= 5)
                        return true;
                }
                return false;
            }
        }
        #endregion
        #region MassSpellReflect()
        public Composite MassSpellReflect()
        {
            return
                new Decorator(ret => GetSpellCooldown("Mass Spell Reflection").TotalSeconds < 1 && doMassSpellRefect,
                    new Sequence(
                        new Action(delegate
                        {
                            Lua.DoString("RunMacroText(\"/equipslot 16 " + "Malevolent Gladiator's Hacker" + "\")");
                            Lua.DoString("RunMacroText(\"/equipslot 17 " + "Masterwork Spiritguard Shield" + "\")");
                            SpellManager.Cast("Mass Spell Reflection");
                            Logging.Write(StyxWoW.Me.HealthPercent + "% - Mass Spell Reflect");
                        }
                        )
                    )
                );
        }

        #endregion
        #region MockingBanner()
        public Composite MockingBanner()
        {
            return new Decorator(
                ret =>
                    StyxWoW.Me.CurrentTarget != null &&
                    IsMovementImpaired(StyxWoW.Me) &&
                    SpellManager.CanCast("Mocking Banner") &&
                    StyxWoW.Me.CurrentTarget.Distance < 25 &&
                    SafeguardTargets25.Count() < 1,
                        new Sequence(
                            new Action(delegate
                            {
                                SpellManager.Cast("Mocking Banner");
                                SpellManager.ClickRemoteLocation(StyxWoW.Me.CurrentTarget.Location);
                                Logging.Write("Casting Mocking Banner: " + StyxWoW.Me.CurrentTarget.Name.ToString());
                            }
                            )
                        )
            );
        }
        #endregion
        #region NeedTasteForBloodDump
        bool NeedTasteForBloodDump
        {
            get
            {
                var tfb = StyxWoW.Me.GetAuraByName("Taste for Blood");
                if (tfb != null)
                {
                    // If we have more than 3 stacks, pop HS
                    if (tfb.TimeLeft.TotalSeconds <= 2 && tfb.StackCount >= 4)
                        return true;

                    // If it's about to drop, and we have at least 2 stacks, then pop HS.
                    // If we have 1 stack, then a slam is better used here.
                    if (tfb.TimeLeft.TotalSeconds <= 2 && tfb.StackCount >= 2)
                        return true;
                }
                return false;
            }
        }
        #endregion
        #region NeedDeadlyCalm
        bool NeedDeadlyCalm
        {
            get
            {
                var tfb = StyxWoW.Me.GetAuraByName("Taste for Blood");
                if (tfb != null)
                {
                    // If we have more than 3 stacks, pop HS
                    if (tfb.TimeLeft.TotalSeconds <= 2 && tfb.StackCount >= 4 && StyxWoW.Me.RagePercent >= 20 && StyxWoW.Me.RagePercent <= 29)
                        return true;

                    // If it's about to drop, and we have at least 2 stacks, then pop HS.
                    // If we have 1 stack, then a slam is better used here.
                    if (tfb.TimeLeft.TotalSeconds <= 2 && tfb.StackCount >= 2 && StyxWoW.Me.RagePercent >= 20 && StyxWoW.Me.RagePercent <= 29)
                        return true;
                }
                return false;
            }
        }
        #endregion
        #region Pummel()

        public Composite Pummel()
        {
            return new Decorator(
                ret =>
                    IsAttackable(StyxWoW.Me.CurrentTarget) &&
                    StyxWoW.Me.CurrentTarget.IsCasting &&
                    StyxWoW.Me.CurrentTarget.CanInterruptCurrentSpellCast &&
                    StyxWoW.Me.CurrentTarget.IsWithinMeleeRange &&
                    StyxWoW.Me.CurrentTarget.CurrentCastTimeLeft.Milliseconds < 1500 &&
                    SpellManager.CanCast("Pummel") &&
                    !IsCrowdControlled(StyxWoW.Me),
                new Action(delegate
                {
                    SpellManager.Cast("Pummel");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Pummeled " + StyxWoW.Me.CurrentTarget.Name + " ms left on cast: " + StyxWoW.Me.CurrentTarget.CurrentCastTimeLeft.Milliseconds);
                }
                )
            );
        }

        #endregion
        #region RallyingCry()
        public Composite RallyingCry()
        {
            return new Decorator(
                ret =>
                    SpellManager.CanCast("Rallying Cry") &&
                    (StyxWoW.Me.HealthPercent <= 25 || (FriendsNeedingHelp30.Count() >= 1)) && //InArena
                    StyxWoW.Me.Combat,
                new Action(delegate
                {
                    SpellManager.Cast("Rallying Cry");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Rallying Cry");
                }
                )
            );
        }
        #endregion
        #region RemoveCC()
        public Composite RemoveCC()
        {
            return
            new Decorator(ret =>
                (StyxWoW.Me.ActiveAuras.ContainsKey("Fear") ||
                StyxWoW.Me.ActiveAuras.ContainsKey("Howl of Terror") ||
                StyxWoW.Me.ActiveAuras.ContainsKey("Intimidating Shout") ||
                StyxWoW.Me.ActiveAuras.ContainsKey("Sap") ||
                StyxWoW.Me.ActiveAuras.ContainsKey("Gouge") ||
                StyxWoW.Me.ActiveAuras.ContainsKey("Repentence") ||
                StyxWoW.Me.ActiveAuras.ContainsKey("Maim") ||
                StyxWoW.Me.ActiveAuras.ContainsKey("Dragon's Breath") ||
                StyxWoW.Me.ActiveAuras.ContainsKey("Sap") ||
                StyxWoW.Me.ActiveAuras.ContainsKey("Psychic Scream")
                ),
                    new PrioritySelector(
                        Cast("Battle Shout", ret => StyxWoW.Me.RagePercent < 25),
                        Cast("Berserker Rage", ret => StyxWoW.Me.RagePercent < 25)
                    )
            );
        }
        #endregion
        #region ShatteringThrow()
        public Composite ShatteringThrow()
        {
            return  new Decorator(
                ret =>
                    IsAttackable(StyxWoW.Me.CurrentTarget) &&
                    StyxWoW.Me.CurrentTarget.Distance <= 30 && 
                    (StyxWoW.Me.CurrentTarget.ActiveAuras.ContainsKey("Ice Block") ||
                    StyxWoW.Me.CurrentTarget.ActiveAuras.ContainsKey("Hand of Protection") ||
                    StyxWoW.Me.CurrentTarget.ActiveAuras.ContainsKey("Divine Shield")) &&
                    SpellManager.CanCast("Shattering Throw") &&
                    StyxWoW.Me.IsFacing(StyxWoW.Me.CurrentTarget),
                        new Action(delegate
                            {
                                SpellManager.Cast("Shattering Throw");
                                Logging.Write(StyxWoW.Me.HealthPercent + "% - Shattering Throw");
                            }
                        )
                );
        }
        #endregion
        #region SweepingStrikes()
        public Composite SweepingStrikes()
        {
            return new Decorator(
                ret =>
                    SpellManager.CanCast("Sweeping Strikes") &&
                    StyxWoW.Me.RagePercent > 30 &&
                    IsAttackable(StyxWoW.Me.CurrentTarget) &&
                    EnemyUnitsInMelee.Count() >= 2 &&
                    EnemyCCInMelee.Count() < 1 &&
                    StyxWoW.Me.CurrentTarget.IsWithinMeleeRange,

                new Action(delegate
                {
                    SpellManager.Cast("Sweeping Strikes");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Sweeping Strikes");
                }
                )
            );
        }
        #endregion
        #region Revenge()
        public Composite Revenge()
        {
            return new Decorator(
                ret =>
                IsAttackable(StyxWoW.Me.CurrentTarget) &&
                StyxWoW.Me.CurrentTarget.IsWithinMeleeRange &&
                StyxWoW.Me.CurrentTarget.HealthPercent > 20 &&
                SpellManager.CanCast("Revenge"),

                new Action(delegate
                {
                    SpellManager.Cast("Revenge");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Revenge");
                }
                )
            );
        }
        #endregion
        #region ShieldBarrier()
        public Composite ShieldBarrier()
        {
            return new Decorator(
                ret =>
                IsAttackable(StyxWoW.Me.CurrentTarget) &&
                StyxWoW.Me.CurrentTarget.IsWithinMeleeRange &&
                StyxWoW.Me.RagePercent >= 60 &&
                StyxWoW.Me.CurrentTarget.MaxMana > 61000 &&
                StyxWoW.Me.CurrentTarget.HealthPercent > 20 &&
                SpellManager.CanCast("Shield Barrier"),

                new Action(delegate
                {
                    SpellManager.Cast("Shield Barrier");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Shield Barrier");
                }
                )
            );
        }
        #endregion
        #region ShieldBlock()
        public Composite ShieldBlock()
        {
            return new Decorator(
                ret =>
                IsAttackable(StyxWoW.Me.CurrentTarget) &&
                StyxWoW.Me.CurrentTarget.IsWithinMeleeRange &&
                StyxWoW.Me.RagePercent >= 60 &&
                StyxWoW.Me.CurrentTarget.MaxMana < 60000 &&
                StyxWoW.Me.CurrentTarget.HealthPercent > 20 &&
                SpellManager.CanCast("Shield Block"),

                new Action(delegate
                {
                    SpellManager.Cast("Shield Block");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Shield Block");
                }
                )
            );
        }
        #endregion
        #region ShieldSlam()
        public Composite ShieldSlam()
        {
            return new Decorator(
                ret =>
                IsAttackable(StyxWoW.Me.CurrentTarget) &&
                StyxWoW.Me.CurrentTarget.IsWithinMeleeRange &&
                StyxWoW.Me.CurrentTarget.HealthPercent > 20 &&
                SpellManager.CanCast("Shield Slam"),

                new Action(delegate
                {
                    SpellManager.Cast("Shield Slam");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Shield Slam");
                }
                )
            );
        }
        #endregion
        #region Shockwave()

        public Composite Shockwave()
        {
            return new Decorator(
                ret =>
                    IsAttackable(StyxWoW.Me.CurrentTarget) &&
                    StyxWoW.Me.CurrentTarget.IsCasting &&
                    StyxWoW.Me.CurrentTarget.Distance <= 10 &&
                    StyxWoW.Me.CurrentTarget.CanInterruptCurrentSpellCast &&
                    StyxWoW.Me.CurrentTarget.CurrentCastTimeLeft.Milliseconds < 1500 &&
                    StyxWoW.Me.IsFacing(StyxWoW.Me.CurrentTarget) &&
                    ((!SpellManager.CanCast("Pummel") && StyxWoW.Me.CurrentTarget.IsWithinMeleeRange) || !StyxWoW.Me.CurrentTarget.IsWithinMeleeRange) &&
                    SpellManager.CanCast("Shockwave") &&
                    !IsCrowdControlled(StyxWoW.Me),
                new Action(delegate
                {
                    SpellManager.Cast("Shockwave");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Shockwaved: " + StyxWoW.Me.CurrentTarget.Name + " ms left on cast: " + StyxWoW.Me.CurrentTarget.CurrentCastTimeLeft.Milliseconds);
                }
                )
            );
        }

        #endregion
        #region SpellReflect()
        public Composite SpellReflect()
        {
            return
                new Decorator(
                    ret => 
                        EnemyCastingOnMeDistance40.Count() >= 1 && 
                        GetSpellCooldown("Spell Reflection").TotalSeconds < 1 &&
                        EnemyCastingOnMeDistance40.FirstOrDefault().CurrentCastTimeLeft.Milliseconds < 1500,
                    new Sequence(
                        new Action(delegate
                        {
                            Lua.DoString("RunMacroText(\"/equipslot 16 " + "Malevolent Gladiator's Hacker" + "\")");
                            Lua.DoString("RunMacroText(\"/equipslot 17 " + "Masterwork Spiritguard Shield" + "\")");
                            SpellManager.Cast("Spell Reflection");
                            Logging.Write(StyxWoW.Me.HealthPercent + "% - Spell Reflect: " + EnemyCastingOnMeDistance40.FirstOrDefault().CastingSpell.Name);
                        }
                        )
                    )
                );
        }

        #endregion
        #region TasteForBloodPause
        bool TasteForBloodPause
        {
            get
            {
                var tfb3 = StyxWoW.Me.GetAuraByName("Taste for Blood");
                if (tfb3 != null)
                {
                    // 
                    if (tfb3.StackCount >= 6)
                        return false;
                }
                return true;
            }
        }
        #endregion
        #region Throw()
        public Composite Throw()
        {
            return new Decorator(
                ret =>
                    IsAttackable(StyxWoW.Me.CurrentTarget) &&
                    StyxWoW.Me.CurrentTarget.InLineOfSight &&
                    !StyxWoW.Me.CurrentTarget.IsWithinMeleeRange &&
                    StyxWoW.Me.CurrentTarget.HealthPercent <= 5,

                new Action(delegate
                {
                    SpellManager.Cast("Throw");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Throw on " + StyxWoW.Me.CurrentTarget.Name + " at " + StyxWoW.Me.CurrentTarget.HealthPercent + "%.");
                }
                )
            );
        }
        #endregion
        #region UseHealthstone()
        public Composite UseHealthstone()
        {
            return new Decorator(
                ret =>
                    StyxWoW.Me.Combat &&
                    StyxWoW.Me.HealthPercent < 25,
                new Action(delegate
                {
                    WoWItem hs = StyxWoW.Me.BagItems.FirstOrDefault(o => o.Entry == 5512);//5512 Healthstone
                    if (hs != null && hs.CooldownTimeLeft.TotalSeconds <= 0)
                    {
                        hs.Use();
                        Logging.Write(StyxWoW.Me.HealthPercent + "% - Healthstone");
                    }

                })
            );
        }
        #endregion
        #region VictoryRush()
        public Composite VictoryRush()
        {
            return new Decorator(
                ret =>
                    IsAttackable(StyxWoW.Me.CurrentTarget) &&
                    SpellManager.CanCast("Victory Rush") &&
                    StyxWoW.Me.CurrentTarget.HealthPercent > 20 &&
                    StyxWoW.Me.CurrentTarget.IsWithinMeleeRange &&
                    StyxWoW.Me.HealthPercent <= 90,

                new Action(delegate
                {
                    SpellManager.Cast("Victory Rush");
                    Logging.Write(StyxWoW.Me.HealthPercent + "% - Victory Rush");
                }
                )
            );
        }
        #endregion

    }
}
