﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Styx;
using Styx.Common;
using Styx.CommonBot;
using Styx.CommonBot.Routines;
using Styx.Helpers;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;

namespace Warena
{
    public partial class Routine
    {
        public IEnumerable<WoWUnit> EnemyCastingCC10
        {
            get
            {

                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWUnit>(true, false).Where(
                        u =>
                            u.IsAlive &&
                            !u.IsMe &&
                            IsAttackable(u) &&
                            !u.IsFriendly &&
                            !u.IsPet &&
                            u.InLineOfSpellSight &&
                            u.IsCasting &&
                            u.Distance <= 10 && 
                            u.CurrentCastTimeLeft.Milliseconds < 1500 && (
                            u.CastingSpell.Name == "Fear" ||
                            u.CastingSpell.Name == "Polymorph" ||
                            u.CastingSpell.Name == "Cyclone" ||
                            u.CastingSpell.Name == "Hex"
                            )
                                  );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWUnit>();
                }
            }
        }
        public IEnumerable<WoWUnit> SafeguardTargets25
        {
            get
            {
                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWUnit>(true, false).Where(
                        u =>
                            !u.IsMe &&
                            u.IsFriendly &&
                            u.InLineOfSight &&
                            u.Distance <= 25
                    );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWUnit>();
                }
            }
        }
        public IEnumerable<WoWPlayer> FriendsNeedingHelp30
        {
            get
            {
                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWPlayer>(true, false).Where(
                        u =>
                            u.IsAlive &&
                            !u.IsMe &&
                            u.IsFriendly &&
                            u.HealthPercent <= 25 &&
                            u.InLineOfSpellSight &&
                            u.Distance <= 30
                    );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWPlayer>();
                }
            }
        }
        public IEnumerable<WoWUnit> EnemyCCInMelee
        {
            get
            {
                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWUnit>(true, false).Where(
                        u =>
                            u.IsAlive &&
                            !u.IsMe &&
                            IsAttackable(u) &&
                            !u.IsFriendly &&
                            !u.IsPet &&
                            u.IsWithinMeleeRange &&
                            !IsCrowdControlled(u)
                            );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWUnit>();
                }
            }
        }
        public IEnumerable<WoWUnit> EnemyCastingDistance10
        {
            get
            {
                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWUnit>(true, false).Where(
                        u =>
                            IsAttackable(u) &&
                            u.IsAlive &&
                            !u.IsMe &&
                            u.IsHostile &&
                            !u.IsPet &&
                            u.InLineOfSpellSight &&
                            u.IsCasting &&
                            u.Distance <= 10
                     );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWUnit>();
                }
            }
        }
        public IEnumerable<WoWPlayer> EnemyDistance8
        {
            get
            {
                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWPlayer>(true, false).Where(
                        u =>
                            !u.IsFriendly || u.IsHostile &&
                            u.IsAlive &&
                            !u.IsMe &&
                            !u.IsPet &&
                            u.InLineOfSpellSight &&
                            u.Distance <= 8
                        );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWPlayer>();
                }
            }
        }
        public IEnumerable<WoWUnit> EnemyCastingOnMeDistance40
        {
            get
            {
                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWUnit>(true, false).Where(
                        u =>
                            IsAttackable(u) &&
                            !u.IsMe &&
                            u.IsTargetingMeOrPet &&
                            u.IsAlive &&
                            !u.IsPet &&
                            u.InLineOfSpellSight &&
                            u.IsCasting &&
                            u.Distance <= 40 && (
                            u.CastingSpell.Name == "Polymorph" ||
                            u.CastingSpell.Name == "Chaos Bolt" ||
                            u.CastingSpell.Name == "Soul Fire" ||
                            u.CastingSpell.Name == "Repentence" ||
                            u.CastingSpell.Name == "Cyclone" ||
                            u.CastingSpell.Name == "Entangling Roots" ||
                            u.CastingSpell.Name == "Fear" ||
                            u.CastingSpell.Name == "Hex"
                            )
                    );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWUnit>();
                }
            }
        }
        public IEnumerable<WoWPlayer> EnemyCastingCrowdControl
        {
            get
            {
                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWPlayer>(true, false).Where(
                        u =>
                            IsAttackable(u) &&
                            !u.IsMe &&
                            !u.IsTargetingMeOrPet &&
                            u.IsAlive &&
                            !u.IsPet &&
                            u.IsCasting &&
                            u.Distance <= 60 && (
                            u.CastingSpell.Name == "Polymorph" ||
                            u.CastingSpell.Name == "Chaos Bolt" ||
                            u.CastingSpell.Name == "Cyclone" ||
                            u.CastingSpell.Name == "Entangling Roots" ||
                            u.CastingSpell.Name == "Fear" ||
                            u.CastingSpell.Name == "Hex"
                            )
                    );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWPlayer>();
                }
            }
        }
        public IEnumerable<WoWPlayer> Friends20
        {
            get
            {
                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWPlayer>(true, false).Where(
                        u =>
                            u.IsAlive &&
                            !u.IsMe &&
                            u.IsFriendly &&
                            u.InLineOfSpellSight &&
                            u.Distance <= 20
                    );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWPlayer>();
                }
            }
        }
        public IEnumerable<WoWUnit> EnemyIsInvulnerable30
        {
            get
            {
                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWUnit>(true, false).Where(
                        u =>
                            IsAttackable(u) &&
                            u.IsAlive &&
                            !u.IsMe &&
                            u.IsHostile &&
                            !u.IsPet &&
                            u.InLineOfSpellSight &&
                            u.Distance <= 30 &&
                            u.HealthPercent <= 75 &&
                            (u.ActiveAuras.ContainsKey("Ice Block") ||
                            u.ActiveAuras.ContainsKey("Hand of Protection") ||
                            u.ActiveAuras.ContainsKey("Divine Shield"))
                     );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWUnit>();
                }
            }
        }
        public bool IsAttackable(WoWUnit unit)
        {
            if (unit == null)
                return false;

            if (!unit.CanSelect || !unit.Attackable)
                return false;

            if (unit.IsHostile || !unit.IsFriendly)
                return true;

            if (unit.IsDead)
                return false;

            if (unit.IsOnTransport)
                return false;

            return true;
        }
        public IEnumerable<WoWUnit> EnemyUnitsInMelee
        {
            get
            {
                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWUnit>(true, false).Where(
                        u =>
                        !u.IsDead && 
                        u.CanSelect && 
                        u.Attackable && 
                        u.IsHostile && 
                        u.IsWithinMeleeRange
                    );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWUnit>();
                }
            }
        }
        public bool CanMeleeInterrupt(WoWUnit unit)
        {
            if (unit != null)
            {
                if (
                    !IsAttackable(unit.CurrentTarget)
                    )
                    return false;

                if (
                    unit.IsHostile && 
                    unit.IsWithinMeleeRange && 
                    unit.IsCasting && 
                    unit.CanInterruptCurrentSpellCast
                    )
                    return true;

            }
            return false;
        }
        public bool CanShoutInterrupt(WoWUnit unit)
        {
            if (unit != null)
            {
                if (EnemyCastingDistance10.Count() > 1)
                //THEN
                if (
                    unit.IsHostile &&
                    !unit.IsWithinMeleeRange &&
                    unit.IsCasting &&
                    unit.CanInterruptCurrentSpellCast
                ) 
                    return true;
            }
            return false;
        }
        public bool IsCrowdControlled(WoWUnit unit)
        {
            if (unit != null)
            {
                Dictionary<string, WoWAura>.ValueCollection auras = unit.Auras.Values;

                return auras.Any(a =>
                           a.IsHarmful && (
                           a.Spell.Mechanic == WoWSpellMechanic.Banished ||
                           a.Spell.Mechanic == WoWSpellMechanic.Disoriented ||
                           a.Spell.Mechanic == WoWSpellMechanic.Charmed ||
                           a.Spell.Mechanic == WoWSpellMechanic.Horrified ||
                           a.Spell.Mechanic == WoWSpellMechanic.Incapacitated ||
                           a.Spell.Mechanic == WoWSpellMechanic.Polymorphed ||
                           a.Spell.Mechanic == WoWSpellMechanic.Sapped ||
                           a.Spell.Mechanic == WoWSpellMechanic.Shackled ||
                           a.Spell.Mechanic == WoWSpellMechanic.Asleep ||
                           a.Spell.Mechanic == WoWSpellMechanic.Frozen ||
                           a.Spell.Mechanic == WoWSpellMechanic.Invulnerable ||
                           a.Spell.Mechanic == WoWSpellMechanic.Invulnerable2 ||
                           a.Spell.Mechanic == WoWSpellMechanic.Turned ||
                           a.Spell.Mechanic == WoWSpellMechanic.Fleeing ||

                           // Really want to ignore hexed mobs.
                           a.Spell.Name == "Hex"));
            }
            return false;
        }
        public bool IsImmune(WoWUnit unit)
        {
            if (unit != null)
            {
                if (unit.ActiveAuras.ContainsKey("Ice Block") || 
                    unit.ActiveAuras.ContainsKey("Hand of Protection") ||
                    unit.ActiveAuras.ContainsKey("Divine Shield") || 
                    unit.ActiveAuras.ContainsKey("Deterrence") || 
                    unit.ActiveAuras.ContainsKey("Cyclone"))
                    return true;
            }
            return false;
        }
        public bool IsInCC(WoWUnit unit)
        {
            if (unit != null)
            {
                    if (unit.IsHostile &&
                       (unit.ActiveAuras.ContainsKey("Repentance") ||
                        unit.ActiveAuras.ContainsKey("Blinding Light") ||
                        unit.ActiveAuras.ContainsKey("Intimidating Shout") ||
                        unit.ActiveAuras.ContainsKey("Freezing Trap")))
                        return true;
            }
            return false;
        }
        public bool IsMovementImpaired(WoWUnit unit)
        {
            if (unit != null)
            {
                Dictionary<string, WoWAura>.ValueCollection auras = unit.Auras.Values;

                return auras.Any(a =>
                           a.IsHarmful && (a.Spell.Mechanic == WoWSpellMechanic.Snared || a.Spell.Mechanic == WoWSpellMechanic.Rooted));
            }
            return false;
        }
        #region EnemiesCastingCrowdControl60
        
        static string[] CrowdControlSpellNames = new string[] { "Chaos Bolt", "Cyclone", "Entangling Roots", "Fear", "Hex", "Polymorph" };
                static IEnumerable<WoWPlayer> EnemiesCastingCrowdControl60 =
            ObjectManager.GetObjectsOfType<WoWPlayer>(true, false)
            .Where(p => p.Distance <= 60 && p.IsHostile && p.IsCasting && CrowdControlSpellNames.Contains(p.CastingSpell.Name)).ToList();

        #endregion
        #region nearbyFriendlies20

        static IEnumerable<WoWPlayer> nearbyFriendlies20 =
            ObjectManager.GetObjectsOfType<WoWPlayer>(true, false)
            .Where(p => p.Distance <= 20 && p.IsFriendly && p.IsAlive).ToList();

        #endregion
        public IEnumerable<WoWPlayer> EnemyHealerTargets40
        {
            get
            {
                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWPlayer>(true, false).Where(
                        u =>
                            u.IsAlive &&
                            !u.IsMe &&
                            IsAttackable(u) &&
                            !u.IsFriendly &&
                            !u.IsPet &&
                            u.InLineOfSpellSight &&
                            u.Distance <= 40 &&
                            u.IsCastingHealingSpell
                    );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWPlayer>();
                }
            }
        }

    }
}
